"use strict";

exports.__esModule = true;
exports.default = cssLiteralVisitor;

var _path = require("path");

var t = _interopRequireWildcard(require("@babel/types"));

var _template = _interopRequireDefault(require("@babel/template"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

var buildImport = (0, _template.default)('require(FILENAME);');

function cssLiteralVisitor(styles, getFilename) {
  if (styles === void 0) {
    styles = [];
  }

  return {
    TaggedTemplateExpression: function TaggedTemplateExpression(path, _ref) {
      var opts = _ref.opts,
          file = _ref.file;
      var _opts$tagName = opts.tagName,
          tagName = _opts$tagName === void 0 ? 'css' : _opts$tagName;
      var node = path.node;

      if (node.tag.name !== tagName || !path.scope.hasGlobal(tagName)) {
        return;
      }

      var parseError = path.buildCodeFrameError('Could not evaluate css. inline css must be statically analyzable');
      var start = node.start,
          end = node.end; // remove the tag and evaluate as a plain template;

      path.replaceWith(node.quasi);

      var _path$evaluate = path.evaluate(),
          confident = _path$evaluate.confident,
          value = _path$evaluate.value;

      if (!confident) {
        throw parseError;
      }

      var style = {
        value: value,
        start: start,
        end: end
      };

      if (getFilename) {
        var hostFile = file.opts.filename;
        style.path = getFilename(opts, hostFile);
        var filename = (0, _path.relative)((0, _path.dirname)(hostFile), style.path);

        if (!filename.startsWith('.')) {
          filename = "./" + filename;
        }

        path.replaceWith(buildImport({
          FILENAME: t.StringLiteral(filename)
        }) // eslint-disable-line new-cap
        );
      }

      styles.push(style);
    }
  };
}